﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Geometry;
using System.Windows.Forms;

namespace LocalBookmarks
{
    [Viking.Common.Command()]    
    class CreateBookmarkCommand : Viking.UI.Commands.Command
    {
        GridVector2 bookmarkPosition;

        FolderUIObj ParentFolder; 

        public CreateBookmarkCommand(Viking.UI.Controls.SectionViewerControl parent, FolderUIObj parentFolder)
            : base(parent)
        {
            //Make the cursor something distinct and appropriate for measuring
            parent.Cursor = Cursors.Cross;
            //Cursor.Hide();

            ParentFolder = parentFolder; 

            this.CommandActive = true;
        }

        public override void OnMouseMove(object sender, MouseEventArgs e)
        {
            this.CommandActive = true;
            base.OnMouseMove(sender,e);
            Parent.Invalidate();
        }

        public override void OnMouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            GridVector2 NewPosition = Parent.ScreenToWorld(e.X, e.Y);

            //Figure out if we are starting a rectangle
            if (e.Button == MouseButtons.Left)
            {
                bookmarkPosition = Parent.ScreenToWorld(e.X, e.Y);
                Execute(); 
            }

            base.OnMouseDown(sender, e);
        }

        protected override void Execute()
        {
            BookmarkUIObj bookmark = new BookmarkUIObj(ParentFolder);
            System.Drawing.Point ControlPostion = Viking.UI.State.ViewerForm.PointToClient(Cursor.Position);
            Geometry.GridVector2 WorldPosition = Viking.UI.State.ViewerControl.ScreenToWorld(ControlPostion.X, ControlPostion.Y);

            bookmark.X = WorldPosition.X;
            bookmark.Y = WorldPosition.Y;
            bookmark.Z = Viking.UI.State.ViewerControl.Section.Number;
            bookmark.Downsample = Viking.UI.State.ViewerControl.Downsample;
            bookmark.Name = "X:" + bookmark.X.ToString(".1") +
                            " Y:" + bookmark.Y.ToString(".1") +
                            " Z:" + bookmark.Z.ToString(".1");

            bookmark.Save(); 

            base.Execute();
        }

        public override void OnMouseLeave(object sender, EventArgs e)
        {
            Cursor.Show();
            this.CommandActive = false;
            Parent.Invalidate();
            base.OnMouseLeave(sender, e);
        }

        public override void OnMouseEnter(object sender, EventArgs e)
        {
            Cursor.Hide();
            this.CommandActive = true;
            Parent.Invalidate();
            base.OnMouseEnter(sender, e);
        }

        public override void OnDeactivate()
        {
            Cursor.Show();
            base.OnDeactivate();
        }

        public override void OnDraw(Microsoft.Xna.Framework.Graphics.GraphicsDevice graphicsDevice, GridRectangle bounds, double DownSample, Microsoft.Xna.Framework.Graphics.BasicEffect basicEffect)
        {
            BookmarkOverlay.DrawCircle(graphicsDevice, basicEffect, this.oldWorldPosition, Global.BookmarkRadius * DownSample, 
                new Microsoft.Xna.Framework.Color(Microsoft.Xna.Framework.Color.Gold.R, 
                                                  Microsoft.Xna.Framework.Color.Gold.G,
                                                  Microsoft.Xna.Framework.Color.Gold.B,
                                                  0.5f)); 
        }
    }
}
