﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Media3D; 
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Viking.VolumeViewModel;

namespace Viking.VolumeView
{
    /// <summary>
    /// Interaction logic for PyramidViewer.xaml
    /// </summary>
    [System.Windows.Markup.ContentProperty("TileMapping")]
    public partial class PyramidViewer : UserControl
    {

        #region Dependancy Property

        public static readonly DependencyProperty TileMappingProperty;

        public TileMappingViewModel TileMapping
        {
            get { return (TileMappingViewModel)GetValue(PyramidViewer.TileMappingProperty); }
            set { SetValue(PyramidViewer.TileMappingProperty, value); }
        }

        private static void OnTileMappingChanged(DependencyObject o, DependencyPropertyChangedEventArgs e) 
        {
            PyramidViewer v = o as PyramidViewer;

            v.ChangedTileMapping(e.NewValue as TileMappingViewModel, e.OldValue as TileMappingViewModel);
        }

        static PyramidViewer()
        {
            PyramidViewer.TileMappingProperty = DependencyProperty.Register("TileMapping",
                                                                        typeof(TileMappingViewModel),
                                                                        typeof(PyramidViewer),
                                                                        new FrameworkPropertyMetadata(null, new PropertyChangedCallback(OnTileMappingChanged)));
        }

        private double LastValidWidth;
        private double LastValidHeight; 

        #endregion

        public PyramidViewer()
        {
            InitializeComponent();

          

        }

        protected override Size MeasureOverride(Size constraint)
        {
            //Figure out how much room we need for the mappings lowest resolution
//            if(TileMapping == null)
                return base.MeasureOverride(constraint);

//            double downsample = TileMapping.Downsample; 
//            if(double.IsInfinity(downsample))
//                downsample = 256;
            
   //         Size s = new Size(TileMapping.Bounds.Width / downsample, TileMapping.Bounds.Height / downsample);
   //         return new Size(Camera.Width, Camera.Width); 
            //return s; 
        }

        protected override Size ArrangeOverride(Size arrangeBounds)
        {

            return base.ArrangeOverride(arrangeBounds);
        }

        public void ChangedTileMapping(TileMappingViewModel newMapping, TileMappingViewModel oldMapping)
        {
            //The first time we load a mapping we want it centered and downsampled so the entire section can be viewed
            bool CenterMapping = false; 

            if (oldMapping != null)
            {
                oldMapping.Tiles.CollectionChanged -= OnTileCollectionChanged;
            }
            else
            {
                CenterMapping = true; 
            }

 //           this.DataContext = newMapping; 

            PyramidVisual.Clear(); 

            if (newMapping != null)
            {
                newMapping.Tiles.CollectionChanged += OnTileCollectionChanged;
                /*
                Binding StatusXBinding = new Binding();
                StatusXBinding.Source = newMapping;
                StatusXBinding.Path = new PropertyPath("VisibleRect");
                StatusXBinding.Converter = new RectToCenterPointConverter();
                StatusXBinding.ConverterParameter = "x";
                StatusXBinding.StringFormat = "{0:F2}";
                this.StatusX.SetBinding(TextBlock.TextProperty, StatusXBinding);

                Binding StatusYBinding = new Binding();
                StatusYBinding.Source = newMapping;
                StatusYBinding.Path = new PropertyPath("VisibleRect");
                StatusYBinding.Converter = new RectToCenterPointConverter();
                StatusYBinding.ConverterParameter = "y";
                StatusYBinding.StringFormat = "{0:F2}";
                this.StatusY.SetBinding(TextBlock.TextProperty, StatusYBinding);

                Binding DownsampleBinding = new Binding();
                DownsampleBinding.Source = newMapping;
                DownsampleBinding.Path = new PropertyPath("Downsample");
                DownsampleBinding.StringFormat = "{0:F2}";
                this.StatusDownsample.SetBinding(TextBlock.TextProperty, DownsampleBinding);

                Binding TileBinding = new Binding();
                TileBinding.Source = newMapping;
                TileBinding.Path = new PropertyPath("Tiles");
                TileBinding.Converter = new TileViewModelToGeometry3DConverter();
                BindingOperations.SetBinding(this.PyramidVisual, Model3DGroup.ChildrenProperty, TileBinding);
                */

                
                if (CenterMapping)
                {
           //         newMapping.VisibleRect = newMapping.Bounds;
           //         newMapping.Downsample = 256;
                    Camera.Position = new Point3D(newMapping.Bounds.Left + (newMapping.Bounds.Width / 2),
                                                  newMapping.Bounds.Top + (newMapping.Bounds.Height / 2),
                                                  Camera.Position.Z); 
                }
                
                
                UpdateTiles(newMapping);
            }

            //
        }

        private void OnTileCollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (e.OldItems != null)
            {
                foreach (TileViewModel t in e.OldItems)
                {
                    PyramidVisual.Remove(t.Model);
                }
            }

            if (e.NewItems != null)
            {
                foreach (TileViewModel t in e.NewItems)
                {
                    PyramidVisual.Add(t.Model);
                }
            }

//            UpdateTiles();
        }
        
        private void UpdateTiles(TileMappingViewModel mapping)
        {
 //           Viewport.Children.Clear();

            PyramidVisual.Clear(); 

//            ModelVisual3D lightModel = new ModelVisual3D(); 
  //          AmbientLight light = new AmbientLight(Color.FromRgb(255, 255,255));
    //        lightModel.Content = light;

            
             
//            Viewport.Children.Add(lightModel); 
            
   //         CreateBackgroundMesh(TileMapping); 


            foreach (TileViewModel t in mapping.Tiles)
            {   
                /*
                ModelUIElement3D elem = new ModelUIElement3D();
                Binding b = new Binding();
                b.Source = t;
                b.Path = new PropertyPath("Model");
                b.NotifyOnSourceUpdated = true; 
                b.NotifyOnTargetUpdated = false;
                elem.Focusable = false;
                elem.IsHitTestVisible = false;

                BindingOperations.SetBinding(elem, ModelUIElement3D.ModelProperty, b);
                */

                PyramidVisual.Add(t.Model);
            }             
        }

        protected void StepCameraDistance(float multiplier)
        {
            double width = Camera.Width; 
            if (multiplier > 0)
                width *= 0.86956521739130434782608695652174;
            else
                width *= 1.15;
            
            if (width < 1)
                width = 1;
            
            Camera.Width = width;
        }

        //A friendlier way of setting camera distance
        public double CameraDownsample
        {
            get { return Camera.Width / LastValidWidth; }
            set
            {
                Camera.Width = LastValidWidth * value;
            }
        }

        
        public Rect ProjectionBounds
        {
            get
            {
                double aspect = LastValidHeight / LastValidWidth;

                double width = Camera.Width;
                double height = Camera.Width * aspect; 

                Rect R = new Rect(Camera.Position.X - (width/2), Camera.Position.Y - (height / 2), Camera.Width, Camera.Width * aspect); 
                
                return R;
            }
        }

        private void Viewport_MouseWheel(object sender, MouseWheelEventArgs e)
        {
            float multiplier = ((float)e.Delta / 120.0f);

            //This seems complicated, but we want the mouse cursor to be pointing at the same 
            //point in the volume before and after the zoom

             
            

            //This is the point the mouse is at...

 //           GeneralTransform3DTo2D WorldToScreen = SectionPlane.TransformToAncestor(this);
            //VisualTreeHelper.HitTest(Viewport, 
            //WorldToScreen(
            
            //GridVector2 BeforeZoomPosition = Parent.ScreenToWorld(e.X, e.Y);

            StepCameraDistance(multiplier);

            /*
            //This is the point the mouse is at after zooming camera...
            GridVector2 AfterZoomPosition = Parent.ScreenToWorld(e.X, e.Y);

            GridVector2 Offset = BeforeZoomPosition - AfterZoomPosition;

            //Move the camera position by the offset.
            GridVector2 CameraLookat = new GridVector2(Parent.CameraLookAt.X, Parent.CameraLookAt.Y);
            GridVector2 NewCameraLookat = CameraLookat + Offset;
            Parent.CameraLookAt = new Vector3((float)NewCameraLookat.X, (float)NewCameraLookat.Y, Parent.CameraLookAt.Z);

            this.Parent.Invalidate();
             */
        }

        private void Camera_Changed(object sender, EventArgs e)
        {
            if (TileMapping != null)
            {
                TileMapping.Downsample = this.CameraDownsample;
                TileMapping.VisibleRect = this.ProjectionBounds; 
            }
        }

        /// <summary>
        /// ActualWidth and ActualHeight are only valid after layout completes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Viewport_LayoutUpdated(object sender, EventArgs e)
        {
            LastValidHeight = Viewport.ActualHeight;
            LastValidWidth = Viewport.ActualWidth; 
        }

        MouseEventArgs OldMouseEventArgs;
        Point OldScreenPosition;
        private void UserControl_MouseMove(object sender, MouseEventArgs e)
        {
            if (TileMapping == null)
                return; 
           

            if (e.LeftButton == MouseButtonState.Pressed)
            {
                if (OldMouseEventArgs == null)
                {
                    OldMouseEventArgs = e;
                    OldScreenPosition = e.GetPosition(Viewport);
                    return;
                }

                Point ScreenPosition = e.GetPosition(Viewport);

                Rect newVisibleRect = TileMapping.VisibleRect;
                Point3D newPosition = Camera.Position;
                newPosition.X += (OldScreenPosition.X - ScreenPosition.X) * CameraDownsample;
                newPosition.Y -= (OldScreenPosition.Y - ScreenPosition.Y) * CameraDownsample;
                //TileMapping.VisibleRect = newVisibleRect; 
                Camera.Position = newPosition;

                OldScreenPosition = ScreenPosition;

                OldMouseEventArgs = e;
            }

        }

        private void PanelInputOverlay_MouseUp(object sender, MouseButtonEventArgs e)
        {
            OldMouseEventArgs = null;
            OldScreenPosition = new Point(); 
        }

    }
}
