﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq; 

namespace Viking.VolumeModel
{
    /// <summary>
    /// Records the data stored in the <pyramid> element in VikingXML, currently a name, path, and list of levels
    /// </summary>
    public class Pyramid
    {
        public readonly string Name;
        public readonly string Path;

        /// <summary>
        /// Maps downsample levels to the path
        /// </summary>
        private SortedList<int, string> LevelsToPaths = new SortedList<int, string>(); 

        public Pyramid(string name, string path)
        {
            this.Name = name;
            this.Path = path; 
        }

        public Pyramid(XElement PyramidElement)
        {
            this.Name = PyramidElement.Attribute("name").Value;
            this.Path = PyramidElement.Attribute("path").Value;

            //Examine the XML document and determine the scale
            IEnumerable<XElement> LevelElements = PyramidElement.Elements().Where(elem => elem.Name.LocalName == "Level");
            foreach (XElement levelElement in LevelElements)
            {
                int Downsample = System.Convert.ToInt32(levelElement.Attribute("Downsample").Value);
                string LevelPath = levelElement.Attribute("path").Value;

                AddLevel(Downsample, LevelPath); 
            }
        }

        public void AddLevel(int level, string path)
        {
            LevelsToPaths.Add(level, path);
        }

        public IEnumerable<int> GetLevels()
        {
            return LevelsToPaths.Keys.ToArray();
        }

        public string PathForLevel(int level)
        {
            if (LevelsToPaths.ContainsKey(level) == false)
                return null;
            
            return LevelsToPaths[level];
        }
    }
}
