﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using Common.DataStructures;

namespace Viking.VolumeModel
{
    public class TileCacheEntry : CacheEntry<string>
    {
        public Tile Tile;

        public TileCacheEntry(string Key, Tile T) : base(Key)
        {
            Tile = T; 
            LastAccessed = DateTime.Now;
            Size = T.Size; 
        }
    }

    /// <summary>
    /// This object manages construction of tile objects. 
    /// It first checks a cache for a tile matching the request.  If not found it creates a new tile object.
    /// </summary>
    public class TileCache : TimeQueueCache<string, TileCacheEntry, Tile, Tile>
    {
        public TileCache()
        {
            this.MaxCacheSize = 2 << 25;
        }

        static protected string TileKey(string textureFileName, string TransformName)
        {
            return textureFileName + " " + TransformName;
        }

        protected override Tile Fetch(TileCacheEntry key)
        {
            key.WasUsedSinceLastCheckpoint = true; 
            return key.Tile; 
        }

        public Tile ConstructTile(string TileUniqueKey,
                                PositionNormalTextureVertex[] verticies,
                                int[] TriangleIndicies,
                                string textureFileName,
                            /*PORT: ViewModel should handle cache names*/ //   string cachedTextureFileName,
                                string TransformName,
                                int downsample,
                                int MipMapLevels, //Should be one, unless it is the minimum downsample level
                                int size)
        {
            //Check to see if this tile is already loaded
            string key = TileUniqueKey;

            Tile tile = new Tile(TileUniqueKey, 
                verticies,
                TriangleIndicies,
                textureFileName,
                //PORT cachedTextureFileName,
                downsample,
                //PORT: MipMapLevels,
                size);

            Add(key, tile);

            return tile;
        }

        protected override TileCacheEntry CreateEntry(string key, Tile value)
        {
            TileCacheEntry entry = new TileCacheEntry(key, value);
            return entry;
        }

        /// <summary>
        /// Remove tile from cache if it exists
        /// </summary>
        /// <param name="tile"></param>
        protected override bool OnRemoveEntry(TileCacheEntry entry)
        {
            //return dictEntries.TryRemove(entry.Key, out entry);
            return true; 
                            
            /* PORT: Removed for port to model
            entry.Tile.AbortRequest();
            entry.Tile.FreeTexture();
            entry.Tile.Dispose();
             */
            //return true; 
        }

        /// <summary>
        /// Aborts all requests for tiles not on the provided list.
        /// Only one of these methods should be running at a time or much of the cache could be deleted
        /// </summary>
        /// <param name="SafeTiles"></param>
        protected override void OnCheckpointFailed(TileCacheEntry entry)
        {
            /* PORT
            lock (LockObj)
            {
                if (dictEntries.ContainsKey(entry.Key))
                {
                    if (entry.Tile.HasTexture == false)
                    {
                        //Nobody is using it, abort the request
                        entry.Tile.AbortRequest();
                        RemoveEntry(entry);
                    }
                }
            }
             */

            OnRemoveEntry(entry); 
        }             
    }
}
