﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Viking.VolumeModel;
using System.Diagnostics;
using System.Windows;
using System.Windows.Input; 

namespace Viking.VolumeViewModel
{
    /// <summary>
    /// Encapsulates a section within the UI
    /// </summary>
    public class SectionViewModel : DependencyObject
    {
        public readonly Section section;

        public string Name { get { return section.Name; } }

        public int Number { get { return section.Number; } }

        public string Path { get { return section.Path; } }

        public string SubPath { get { return section.SectionSubPath; } }

        public override string ToString()
        {
            return section.ToString(); 
        }

        public string DefaultChannel { get { return section.DefaultChannel; } }
        public string[] Channels { get { return section.Channels; } }

        public string DefaultPyramidTransform { get { return section.DefaultPyramidTransform; } }
        public string DefaultPyramid { get { return section.DefaultPyramid; } }
        public List<string> TilesetNames { get { return section.TilesetNames; } }
        public List<string> PyramidTransformNames { get { return section.PyramidTransformNames; } }
        public SortedList<string,Pyramid> ImagePyramids { get { return section.ImagePyramids; } }

        /// <summary>
        /// The currently displayed channels
        /// </summary>
        public ChannelInfo[] ChannelInfoArray {
            get { return section.ChannelInfoArray;}
            set { section.ChannelInfoArray = value; }
        }

        /// <summary>
        /// The names of all channels supported by this section
        /// </summary>
        public List<string> ChannelNames { get { return section.ChannelNames; } }

        private VolumeViewModel _VolumeViewModel;
        public VolumeViewModel VolumeViewModel { get { return _VolumeViewModel; } }

        public TileMappingViewModel DefaultMapping
        {
            get
            {
                MappingBase mapping = MappingManager.GetMapping(_VolumeViewModel.DefaultVolumeTransform, section, DefaultChannel, DefaultPyramidTransform);
                TileMappingViewModel mapViewModel = new TileMappingViewModel(mapping);
                return mapViewModel;
            }
        }

#region Dependency Properties

        private static readonly DependencyProperty MappingProperty;

        public MappingBase Mapping
        {
            get { return (MappingBase)this.GetValue(MappingProperty); }
            set { this.SetValue(MappingProperty, value); }
        }

        static SectionViewModel()
        {
            SectionViewModel.MappingProperty = DependencyProperty.Register("Mapping",
                                                                                   typeof(MappingBase),
                                                                                   typeof(SectionViewModel),
                                                                                   new FrameworkPropertyMetadata(null,
                                                                                       FrameworkPropertyMetadataOptions.AffectsRender));
        }

#endregion 

        public SectionViewModel(VolumeViewModel Volume, Section section)
        {
            this._VolumeViewModel = Volume;
            this.section = section; 
        }

        public void PrepareTransform(string transform)
        {
            this.section.PrepareTransform(transform); 
        }

        #region Commands

            private bool CanChangeSectionNumber(int delta)
            {
                return false;
            }

            private void ChangeSectionNumber(int delta)
            {

            }

            public Microsoft.Practices.Prism.Commands.DelegateCommand<int> IncrementZ
            {
                get
                {
                    return new Microsoft.Practices.Prism.Commands.DelegateCommand<int>(f => this.ChangeSectionNumber(1) );
                }
            }

            public Microsoft.Practices.Prism.Commands.DelegateCommand<int> DecrementZ
            {
                get
                {
                    return new Microsoft.Practices.Prism.Commands.DelegateCommand<int>(f => this.ChangeSectionNumber(-1));
                }
            }

        #endregion
         
    }
}
