﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Viking.Common;
using System.Windows.Forms;
using WebAnnotationModel;
using System.Diagnostics; 

namespace WebAnnotation.ViewModel
{
    /// <summary>
    /// This class represents a link between locations. This object is a little unique because it is
    /// not tied to the database object like the other *obj classes
    /// </summary>
    public class LocationLink : Viking.Objects.UIObjBase
    {
        public override int GetHashCode()
        {
            return A.GetHashCode();
        }

        public override bool Equals(object obj)
        {
            LocationLink link = obj as LocationLink;
            if (link == null)
                return false;

            return (link.A.ID == A.ID && link.B.ID == B.ID); 
        }

        public override string ToString()
        {
            return A.ID.ToString() + " <-> " + B.ID.ToString() + " Sections: " + minSection.ToString() + "-" + maxSection.ToString();
        }
        
        /// <summary>
        /// LocationOnSection is the location on the section being viewed
        /// </summary>
        public Location A; 

        /// <summary>
        /// LocationOnSection is the location on the section being viewed
        /// </summary>
        public Location B;

        public int minSection { get { return A.Section < B.Section ? A.Section : B.Section; } }
        public int maxSection { get { return A.Section > B.Section ? A.Section : B.Section; } }
        
        public RoundLineCode.RoundLine lineGraphic;
        public Geometry.GridLineSegment lineSegment;

        public double Radius
        {
            get
            {
                return Math.Min(A.Radius,B.Radius) / 4.0f; 
            }
        }

        public LocationLink(Location LocOne, Location LocTwo)
        {
            Debug.Assert(LocOne != LocTwo);
            this.A = LocOne.ID < LocTwo.ID ? LocOne : LocTwo;
            this.B = LocOne.ID > LocTwo.ID ? LocOne : LocTwo; 

            lineSegment = new Geometry.GridLineSegment(A.VolumePosition, B.VolumePosition);
            lineGraphic = new RoundLineCode.RoundLine((float)lineSegment.A.X,
                                                      (float)lineSegment.A.Y,
                                                      (float)lineSegment.B.X,
                                                      (float)lineSegment.B.Y);
        }

        #region IUIObjectBasic Members
        
        public override System.Windows.Forms.ContextMenu ContextMenu
        {
            get
            {
                ContextMenu menu = new ContextMenu();

                MenuItem menuSeperator = new MenuItem(); 
                MenuItem menuDelete = new MenuItem("Delete", ContextMenu_OnDelete);

                menu.MenuItems.Add(menuSeperator); 
                menu.MenuItems.Add(menuDelete); 

                return menu; 
            }
        }

        public override string ToolTip
        {
            get { return A.ID.ToString() + " -> " + B.ID.ToString(); }
        }

        public override void Save()
        {
            throw new NotImplementedException();
        }

        #endregion

        protected void ContextMenu_OnDelete(object sender, EventArgs e)
        {
            Delete();
        }

        public override void Delete()
        {
            CallBeforeDelete(); 

            Store.LocationLinks.DeleteLink(this.A.ID, this.B.ID);

            CallAfterDelete();
        }

    }
}
