﻿using System;
using System.ComponentModel; 
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Common.UI; 
using Viking.Common;
using System.Windows.Forms;
using System.Drawing;
using WebAnnotationModel;

namespace WebAnnotation.ViewModel
{
    public class Structure : Viking.Objects.UIObjBase, IEquatable<Structure>, IEqualityComparer<Structure>
    {
        public StructureObj modelObj;

        public override string ToString()
        {
            return modelObj.ToString();
        }

        public override int GetHashCode()
        {
            return modelObj.GetHashCode();
        }

        public override bool Equals(object obj)
        {
            Structure Obj = obj as Structure;
            if (Obj != null)
            {
                return modelObj.Equals(Obj.modelObj);
            }

            StructureObj Obj2 = obj as StructureObj;
            if (Obj2 != null)
            {
                return modelObj.Equals(Obj2);
            }

            return false;
        }

        public Structure(StructureObj data)
        {
            this.modelObj = data; 
        }

        public Structure Parent
        {
            get
            {
                if (modelObj.Parent == null)
                    return null;

                return new Structure(modelObj.Parent);
            }
        }
        
        [Column("ID")]
        public long ID
        {
            get { return modelObj.ID; }
        }

        public string Notes
        {
            get { return modelObj.Notes; }
            set
            {
                modelObj.Notes = value;
            }
        }

        [Column("Verified")]
        public bool Verified
        {
            get { return modelObj.Verified; }
            set
            {
                modelObj.Verified = value;
            }
        }

        [Column("Confidence")]
        public double Confidence
        {
            get { return modelObj.Confidence; }
            set
            {
                modelObj.Confidence = value;
            }
        }

        public string[] Tags
        {
            get { return modelObj.Tags; }
            set
            {
                modelObj.Tags = value;
            }
        }

        [Column("Label")]
        public string InfoLabel
        {
            get { return modelObj.Label; }
            set
            {
                modelObj.Label = value;
            }
        }

        [Column("Type")]
        public StructureType Type
        {
            get
            {
                return new StructureType(modelObj.Type); 
            }
        }

        #region IUIObject Members : IUIObject

        public new event PropertyChangedEventHandler ValueChanged
        {
            add { modelObj.PropertyChanged += value; }
            remove { modelObj.PropertyChanged -= value; }
        }

        public override ContextMenu ContextMenu
        {
            get
            {
                ContextMenu menu = new ContextMenu();

                menu.MenuItems.Add("Delete", ContextMenu_OnDelete);
                menu.MenuItems.Add("Properties", ContextMenu_OnProperties);

                return menu;
            }
        }
        
        public override void Save()
        {
            Store.Structures.Save();
        }

        public override Viking.UI.Controls.GenericTreeNode CreateNode()
        {
            return new Viking.UI.Controls.GenericTreeNode(this);
        }

        public override Type[] AssignableParentTypes
        {
            get { return new System.Type[] { typeof(StructureObj) }; }
        }



        #endregion

        protected void ContextMenu_OnProperties(object sender, EventArgs e)
        {
            Viking.UI.Forms.PropertySheetForm.Show(this);
        }

        protected void ContextMenu_OnDelete(object sender, EventArgs e)
        {
            Delete();
        }

        public override void Delete()
        {
            Store.Structures.Remove(modelObj);

            /*
            Structure OriginalParent = this.Parent;
            this.Parent = null;

            DBACTION originalAction = this.DBAction;
            this.DBAction = DBACTION.DELETE;

            bool success = Store.Structures.Save();
            if (!success)
            {
                //Write straight to data since we have an assert to check whether an object is being deleted, but
                //in this case we know it is ok
                this.Data.DBAction = originalAction;
                this.Parent = OriginalParent;
            }
            */
        }

        bool IEquatable<Structure>.Equals(Structure other)
        {
            return this.modelObj.ID == other.modelObj.ID; 
        }

        public bool Equals(Structure x, Structure y)
        {
            return x.modelObj.ID == y.modelObj.ID; 
        }

        public int GetHashCode(Structure obj)
        {
            return obj.modelObj.GetHashCode(); 
        }
    }
}
