﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized; 
using System.Collections.Concurrent; 
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.ServiceModel; 

using WebAnnotationModel.Service;
using WebAnnotationModel.Objects;

namespace WebAnnotationModel
{
    public class StructureTypeStore : StoreBaseWithIndexKeyAndParent<AnnotateStructureTypesClient,
                                         IAnnotateStructureTypes,
                                        long, 
                                        LongIndexGenerator,
                                        StructureTypeObj,
                                         StructureType>
    {
        #region Proxy

        protected override AnnotateStructureTypesClient CreateProxy()
        {
            AnnotateStructureTypesClient proxy =  new Service.AnnotateStructureTypesClient("Annotation.Service.Interfaces.IAnnotateStructureTypes-Binary", State.EndpointAddress);
            proxy.ClientCredentials.UserName.UserName = State.UserCredentials.UserName;
            proxy.ClientCredentials.UserName.Password = State.UserCredentials.Password;
            return proxy; 
        }

        
            /*
        public override StructureTypeObj Create()
        {
            StructureTypeObj newObj = new StructureTypeObj();
            InternalAdd(newObj); 

            AnnotateStructureTypesClient proxy = null;
            try
            {
                proxy = CreateProxy();
                proxy.Open();

                long OriginalID = newObj.ID;

                long[] newIDs = proxy.UpdateStructureTypes(new StructureType[] { newObj.GetData() });

                

                Store.Locations.InternalDelete(newObj.ID);

                newObj.GetData().ID = newIDs[1];
                newObj.GetData().ParentID = newIDs[0];
                newObj.DBAction = DBACTION.NONE;

                Store.StructureTypes.InternalAdd(newObj);

            }
            catch (Exception e)
            {
                ShowStandardExceptionMessage(e);
                InternalDelete(newObj.ID);
                return null;
            }
            finally
            {
                if (proxy != null)
                    proxy.Close();
            }
             
            return newObj; 
        }
    */
        
        protected override long[] ProxyUpdate(AnnotateStructureTypesClient proxy, StructureType[] objects)
        {
            return proxy.UpdateStructureTypes(objects);
        }

        protected override StructureType ProxyGetByID(AnnotateStructureTypesClient proxy, long ID)
        {
            return proxy.GetStructureTypeByID(ID);
        }

        protected override StructureType[] ProxyGetByIDs(AnnotateStructureTypesClient proxy, long[] IDs)
        {
            return proxy.GetStructureTypesByIDs(IDs);
        }

        protected override StructureType[] ProxyGetBySectionCallback(out long TicksAtQueryExecute, out long[] DeletedLocations, StoreBaseWithKey<AnnotateStructureTypesClient, IAnnotateStructureTypes, long, StructureTypeObj, StructureType>.GetObjectBySectionCallbackState state, IAsyncResult result)
        {
            throw new NotImplementedException();
        }

        protected override ConcurrentDictionary<long, StructureTypeObj> ProxyBeginGetBySection(AnnotateStructureTypesClient proxy,
                                                             long SectionNumber,
                                                             DateTime LastQuery,
                                                             AsyncCallback callback,
                                                             object asynchState)
        {
            throw new NotImplementedException();
        }

        #endregion

        public override void  Init()
        {
 	        LoadStructureTypes();
        }

        /// <summary>
        /// At startup we load the entire structure types table since it is fairly static
        /// </summary>
        public void LoadStructureTypes()
        {
            StructureType[] types  = new StructureType[0]; 
            AnnotateStructureTypesClient proxy = null;
            try
            {
                proxy = CreateProxy();
                proxy.Open();
                types = proxy.GetStructureTypes();
            }
            catch (Exception e)
            {
                ShowStandardExceptionMessage(e);
            }
            finally
            {
                if(proxy != null)
                    proxy.Close();
            }

            //Populate our cache
            StructureTypeObj[] objList = new StructureTypeObj[types.Length];
            //Dictionary<long, StructureType> TypesTable = new Dictionary<long, StructureType>(types.Length);

            for (int i = 0; i < types.Length; i++)
            {
                objList[i] = new StructureTypeObj(types[i]);
                //Do this first so we don't ask the server for parents we've already downloaded
                //IDToObject.Add(typeObj.ID, typeObj);

                //InternalAdd(typeObj, false); 
            }

            InternalAdd(objList, false); 
            //Populate Parent/Child relationships

            /*
            foreach (StructureTypeObj typeObj in objList)
            {
                Add(typeObj); 
            }
             */
        }
    }
}
