// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// File         : AsyncMosaicSave.h
// Author       : Bradley C. Grimm
// Created      : 2009/06/25
// Copyright    : (C) 2009 University of Utah
// License      : GPLv2
// Description  : An asyncronous save manager so that ir-assemble
//                progress is not halted with each save.

#ifndef __ASYNC_MOSAIC_SAVE_H__
#define __ASYNC_MOSAIC_SAVE_H__

#include <thread/the_boost_thread.hxx>

//----------------------------------------------------------------
// save_mosaic_t
// 
template <typename T>
class save_mosaic_t : public the_transaction_t
{
public:
  save_mosaic_t(const T * image, 
                const char * filename,
                bool blab = true):
    m_image(image),
    m_filename(filename),
    m_blab(blab)
  {
  }
  
  // virtual:
  void execute(the_thread_interface_t * thread)
  {
    typedef typename itk::ImageFileWriter<T> writer_t;
    typename writer_t::Pointer writer = writer_t::New();
    writer->SetInput(m_image);
    
    if (m_blab) cout << "saving " << m_filename << endl;
    writer->SetFileName(m_filename);
    writer->Update();
  }
  
  // image:
  typename T::ConstPointer m_image;
  
  // filename:
  the_text_t m_filename;

  bool m_blab;
};


template <typename T> 
void save_mosaic_async(const T * image, 
                       const the_text_t & filename, 
                       the_thread_pool_t &thread_pool,
                       bool blab = true)
{
  std::list<the_transaction_t *> schedule;
  save_mosaic_t<T> * t = new save_mosaic_t<T>(image, filename, blab);
  schedule.push_back(t);
  
  // setup the thread pool:
  thread_pool.push_back(schedule);
  //thread_pool.pre_distribute_work();
  thread_pool.start();
}

#endif


