#if IDX_ENABLED

#include <visus_support.h>

///////////////////////////////////////////////////////
void cleanup_idx(VisusFile *vf, VisusQuery *query, VisusAccess *access)
{
  VisusDestroyQuery(vf,query);
  VisusDestroyAccess(access);
  VisusDestroy(vf);
}

///////////////////////////////////////////////////////
int save_to_idx(const char *filename, const native_image_t *image, 
                const int z_slice, const int channel)
{
  if (!image)                            { cerr << "Error: image is NULL\n";              return -1; }
  if (!filename || strlen(filename)==0)  { cerr << "Error: filename is empty\n";          return -1; }
  VisusFile* vf=VisusOpen(filename);
  if (!vf)                               { cerr << "Error: cannot open "<<filename<<endl; return -1; }
  VisusPoint dims=VisusGetBoxDimension(vf->box);
  int width =image->GetLargestPossibleRegion().GetSize()[0];
  int height=image->GetLargestPossibleRegion().GetSize()[1];

  //make sure z_slice, channel and image dimensions are valid
  if (vf->nfields<=channel || channel<0) { cerr << "Error: invalid field specified\n";    return -1; }
  if (dims.z <= z_slice    || z_slice<0) { cerr << "Error: invalid slice specified\n";    return -1; }
  if (!(dims.x==width && dims.y==height)){ cerr << "Error: idx/image dimensions differ\n";return -1; }

  //this is the bounding box of the region I'm going to write
  VisusBox slice_box=vf->box;
  PGET(slice_box.p1,2)=z_slice;
  PGET(slice_box.p2,2)=z_slice;
  VisusField  *field =&vf->fields[channel];
  VisusAccess *access=VisusCreateAccess(vf);

  //create query
  VisusQuery* write_query=VisusCreateQuery(vf,slice_box,VisusGetMaxH(vf),0,VisusGetMaxH(vf));
  if (!write_query || write_query->totsamples!=width*height) 
  {
    cerr << "Error: cannot create write query\n"; 
    cleanup_idx(vf,write_query,access); 
    return -1; 
  }

  //get data size
  int slice_data_size=VisusGetBufferSize(vf,field,write_query->totsamples);
  if (slice_data_size != width*height) 
  { 
    cerr << "Error: image size and query size do not match.\n\t"
         << "Please check that field data type matches image data type.\n"; 
    return -1; 
  }

  //write image
  VisusWrite(vf,access,write_query,field,vf->time_from,VisusCreateData(slice_data_size,const_cast<unsigned char*>(image->GetBufferPointer()),0,0));
  if (write_query->status!=VISUS_QUERY_FINISHED)      
  {
    cerr << "Error: write data failed\n"; 
    cleanup_idx(vf,write_query,access); 
    return -1; 
  }

  //close idx
  cleanup_idx(vf, write_query, access);

  return 0;
}

#endif // IDX_ENABLED

